@@ -1,3 +1,4 @@
+//go:build stm32f4disco
 // +build stm32f4disco
 
 package machine
@@ -8,16 +9,40 @@ import (
 )
 
 const (
-	LED         = LED_BUILTIN
 	LED1        = LED_GREEN
 	LED2        = LED_ORANGE
 	LED3        = LED_RED
 	LED4        = LED_BLUE
-	LED_BUILTIN = LED_GREEN
 	LED_GREEN   = PD12
 	LED_ORANGE  = PD13
 	LED_RED     = PD14
 	LED_BLUE    = PD15
+	LED         = LED_BUILTIN
+	LED_BUILTIN = LED_GREEN
+)
+
+const (
+	BUTTON = PA0
+)
+
+// Analog Pins
+const (
+	ADC0  = PA0
+	ADC1  = PA1
+	ADC2  = PA2
+	ADC3  = PA3
+	ADC4  = PA4
+	ADC5  = PA5
+	ADC6  = PA6
+	ADC7  = PA7
+	ADC8  = PB0
+	ADC9  = PB1
+	ADC10 = PC0
+	ADC11 = PC1
+	ADC12 = PC2
+	ADC13 = PC3
+	ADC14 = PC4
+	ADC15 = PC5
 )
 
 // UART pins
@@ -27,17 +52,19 @@ const (
 )
 
 var (
-	UART0 = UART{
-		Buffer:          NewRingBuffer(),
-		Bus:             stm32.USART2,
-		AltFuncSelector: AF7_USART1_2_3,
+	UART1  = &_UART1
+	_UART1 = UART{
+		Buffer:            NewRingBuffer(),
+		Bus:               stm32.USART2,
+		TxAltFuncSelector: AF7_USART1_2_3,
+		RxAltFuncSelector: AF7_USART1_2_3,
 	}
-	UART1 = &UART0
+	DefaultUART = UART1
 )
 
 // set up RX IRQ handler. Follow similar pattern for other UARTx instances
 func init() {
-	UART0.Interrupt = interrupt.New(stm32.IRQ_USART2, UART0.handleInterrupt)
+	UART1.Interrupt = interrupt.New(stm32.IRQ_USART2, _UART1.handleInterrupt)
 }
 
 // SPI pins
@@ -73,7 +100,7 @@ const (
 )
 
 var (
-	I2C0 = I2C{
+	I2C0 = &I2C{
 		Bus:             stm32.I2C1,
 		AltFuncSelector: AF4_I2C1_2_3,
 	}
