@@ -1,3 +1,4 @@
+//go:build avr && atmega1280
 // +build avr,atmega1280
 
 package machine
@@ -84,10 +85,10 @@ const (
 	PK1 = portK + 1
 	PK2 = portK + 2
 	PK3 = portK + 3
-	PK4 = portH + 4
-	PK5 = portH + 5
-	PK6 = portH + 6
-	PK7 = portH + 7
+	PK4 = portK + 4
+	PK5 = portK + 5
+	PK6 = portK + 6
+	PK7 = portK + 7
 	PL0 = portL + 0
 	PL1 = portL + 1
 	PL2 = portL + 2
@@ -179,6 +180,9 @@ func (pwm PWM) Configure(config PWMConfig) error {
 			avr.TCCR0B.Set(prescaler)
 			// Set the PWM mode to fast PWM (mode = 3).
 			avr.TCCR0A.Set(avr.TCCR0A_WGM00 | avr.TCCR0A_WGM01)
+			// monotonic timer is using the same time as PWM:0
+			// we must adust internal settings of monotonic timer when PWM:0 settings changed
+			adjustMonotonicTimer()
 		} else {
 			avr.TCCR2B.Set(prescaler)
 			// Set the PWM mode to fast PWM (mode = 3).
@@ -714,6 +718,9 @@ func (pwm PWM) Set(channel uint8, value uint32) {
 				avr.TCCR0A.SetBits(avr.TCCR0A_COM0B1)
 			}
 		}
+		// monotonic timer is using the same time as PWM:0
+		// we must adust internal settings of monotonic timer when PWM:0 settings changed
+		adjustMonotonicTimer()
 	case 1:
 		mask := interrupt.Disable()
 		switch channel {
