@@ -1,3 +1,4 @@
+//go:build tinygo.riscv && virt && qemu
 // +build tinygo.riscv,virt,qemu
 
 package runtime
@@ -15,17 +16,13 @@ type timeUnit int64
 
 var timestamp timeUnit
 
-func postinit() {}
-
 //export main
 func main() {
 	preinit()
 	run()
-	abort()
+	exit(0)
 }
 
-const asyncScheduler = false
-
 func ticksToNanoseconds(ticks timeUnit) int64 {
 	return int64(ticks)
 }
@@ -51,16 +48,35 @@ var (
 	// UART0 output register.
 	stdoutWrite = (*volatile.Register8)(unsafe.Pointer(uintptr(0x10000000)))
 	// SiFive test finisher
-	testFinisher = (*volatile.Register16)(unsafe.Pointer(uintptr(0x100000)))
+	testFinisher = (*volatile.Register32)(unsafe.Pointer(uintptr(0x100000)))
 )
 
 func putchar(c byte) {
 	stdoutWrite.Set(uint8(c))
 }
 
+func getchar() byte {
+	// dummy, TODO
+	return 0
+}
+
+func buffered() int {
+	// dummy, TODO
+	return 0
+}
+
 func abort() {
+	exit(1)
+}
+
+func exit(code int) {
 	// Make sure the QEMU process exits.
-	testFinisher.Set(0x5555) // FINISHER_PASS
+	if code == 0 {
+		testFinisher.Set(0x5555) // FINISHER_PASS
+	} else {
+		// Exit code is stored in the upper 16 bits of the 32 bit value.
+		testFinisher.Set(uint32(code)<<16 | 0x3333) // FINISHER_FAIL
+	}
 
 	// Lock up forever (as a fallback).
 	for {
